﻿using System.Collections.Generic;
using System.Linq;

namespace ProductExchangeModel.BdcModel1
{
    class DataAccessLayer
    {
        /// <summary>
        /// Get Products from 'External System'
        /// </summary>
        /// <param name="limitParameter">Limit Filter value</param>
        /// <param name="wildcardParameter">Wildcard Filter value</param>
        /// <returns>Collection of matching records</returns>
        internal static IEnumerable<ExternalSytemProduct> GetProductsFromExternalSystem(int limitParameter, string wildcardParameter)
        {
            const int noValuePassed = 0;
            const int defaultLimit = 100;

            // Use default value if not specified
            if (limitParameter == noValuePassed)
            {
                limitParameter = defaultLimit;
            }

            var items = new List<ExternalSytemProduct>(5)
                            {
                                new ExternalSytemProduct(680, "Road Frame - Black",(decimal) 1431.50),
                                new ExternalSytemProduct(706, "Road Frame - Red", (decimal) 1211.90),
                                new ExternalSytemProduct(707, "Sport-100 Helmet - Red", (decimal) 34.99),
                                new ExternalSytemProduct(708, "Sport-100 Helmet - Black", (decimal) 30.25),
                                new ExternalSytemProduct(709, "Mountain Bike Socks - White", (decimal) 9.50)
                            };

            // Remove the * to get inner text for filter comparision
            string filterText = wildcardParameter.Replace("*", "");

            // Contains Method
            if (wildcardParameter.StartsWith("*") & wildcardParameter.EndsWith("*"))
            {
                return items.Where(p => p.Name.Contains(filterText)).Take(limitParameter);
            }
            // Ends With
            if (wildcardParameter.StartsWith("*"))
            {
                return items.Where(p => p.Name.EndsWith(filterText)).Take(limitParameter);
            }
            // Starts With
            if (wildcardParameter.EndsWith("*"))
            {
                return items.Where(p => p.Name.StartsWith(filterText)).Take(limitParameter);
            }

            // Equals
            return items.Where(p => p.Name == filterText).Take(limitParameter);
        }

        /// <summary>
        /// Get Single Product by identifier from 'External System'
        /// </summary>
        /// <param name="inParameter">The identifier</param>
        /// <returns>Single Product</returns>
        internal static ExternalSytemProduct GetSingleProductFromExternalSystem(int inParameter)
        {
            return GetProductsFromExternalSystem(0, null).Single(p => p.ProductID == inParameter);
        }

        /// <summary>
        /// Example of record from 'External System' where
        /// it does not fully reperesent what we want in
        /// SharePoint
        /// </summary>
        internal class ExternalSytemProduct
        {
            public ExternalSytemProduct(int productId, string name, decimal price)
            {
                Name = name;
                ProductID = productId;
                ListPrice = price;
            }

            public int ProductID { get; set; }
            public string Name { get; set; }
            public decimal ListPrice { get; set; }
        }
    }
}
