﻿using System.Collections.Generic;
using System.Linq;

namespace ProductExchangeModel.BdcModel1
{
    /// <summary>
    /// Service Class for returning records from External System
    /// </summary>
    public class ProductExchangeRatesService
    {
        /// <summary>
        /// Finder Method
        /// </summary>
        /// <param name="LimitParameter">Limit Filter</param>
        /// <param name="WildcardParameter">Wilcard</param>
        /// <returns>Collection of Products matching filters</returns>
        public static IEnumerable<Product> GetAllProducts(int LimitParameter, string WildcardParameter)
        {
            // Get the items from the External System(s)
            IEnumerable<DataAccessLayer.ExternalSytemProduct> externalSystemProducts =
                DataAccessLayer.GetProductsFromExternalSystem(LimitParameter, WildcardParameter);

            // Get Exchange rate for pound sterling
            decimal poundSterlingExchangeRate = CurrencyHelper.GetExchangeRate(Currency.PoundSterling);

            // Get the Exchange rate for Yen
            decimal yenExchangeRate = CurrencyHelper.GetExchangeRate(Currency.Yen);

            // Return collection of Products created from External Data and own values
            return externalSystemProducts.Select(externalSystemProduct => new Product
                                                                              {
                                                                                  ProductId =
                                                                                      externalSystemProduct.ProductID,
                                                                                  Name = externalSystemProduct.Name,
                                                                                  Price =
                                                                                      externalSystemProduct.ListPrice,
                                                                                  PriceInPoundSterling =
                                                                                      externalSystemProduct.ListPrice*
                                                                                      poundSterlingExchangeRate,
                                                                                  PriceInYen =
                                                                                      externalSystemProduct.ListPrice*
                                                                                      yenExchangeRate
                                                                              });
        }


        /// <summary>
        /// Specific Finder Method
        /// </summary>
        /// <param name="InParameter">Identifier</param>
        /// <returns>Single record with matching Identifier</returns>
        public static Product GetSingleProduct(int InParameter)
        {
            // Get the items from the External System(s)
            DataAccessLayer.ExternalSytemProduct externalSystemProduct =
                DataAccessLayer.GetSingleProductFromExternalSystem(InParameter);

            // Get Exchange rate for pound sterling
            decimal poundSterlingExchangeRate = CurrencyHelper.GetExchangeRate(Currency.PoundSterling);

            // Get the Exchange rate for Yen
            decimal yenExchangeRate = CurrencyHelper.GetExchangeRate(Currency.Yen);

            // Return collection of Products created from External Data and own values
            return new Product
                       {
                           ProductId = externalSystemProduct.ProductID,
                           Name = externalSystemProduct.Name,
                           Price = externalSystemProduct.ListPrice,
                           PriceInPoundSterling = externalSystemProduct.ListPrice*poundSterlingExchangeRate,
                           PriceInYen = externalSystemProduct.ListPrice*yenExchangeRate
                       };
        }
    }
}